/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <linux/platform_device.h>
#include <linux/io.h>
#include <mach/hardware.h>
#include <mach/common.h>
#include <mach/audmux.h>
#include <mach/armadillo400_wm8978.h>
#include "armadillo400_iomux.h"

static struct pad_desc armadillo400_audio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_44_AUD5_TXD)
	MX25_PAD_KPP_COL0__AUD5_TXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_DOWN),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_AUD5_RXD)
	MX25_PAD_KPP_COL1__AUD5_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_AUD5_TXC)
	MX25_PAD_KPP_COL2__AUD5_TXC(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_AUD5_TXFS)
	MX25_PAD_KPP_COL3__AUD5_TXFS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_AUD5_RXC)
	MX25_PAD_KPP_ROW2__AUD5_RXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_AUD5_RXFS)
	MX25_PAD_KPP_ROW3__AUD5_RXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif

#if defined(CONFIG_ARMADILLO400_CON9_21_AUD6_TXD)
	MX25_PAD_CSI_MCLK__AUD6_TXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_DOWN),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_AUD6_RXD)
	MX25_PAD_CSI_VSYNC__AUD6_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_AUD6_TXC)
	MX25_PAD_CSI_HSYNC__AUD6_TXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_AUD6_TXFS)
	MX25_PAD_CSI_PIXCLK__AUD6_TXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_AUD6_RXC)
	MX25_PAD_CSI_D8__AUD6_RXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_AUD6_RXFS)
	MX25_PAD_CSI_D9__AUD6_RXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

/*
 * SSI(Sync)    | PortA(Async) | PortB(Async) | PAD     | Codec
 * -------------+--------------+--------------+---------+-------
 * STCK         | TXC  ->      | TXC  ->      | TXC  -> | BCLK
 * STFS         | TXFS ->      | TXFS ->      | TXFS -> | LRCIN
 * SYS_CLK(SRCK)| RXC  ->      | RXC  ->      | RXC  -> | MCLK
 * SRFS         | RXFS ->      | RXFS <-      | RXFS <- | LRCOUT
 * STXD         | TXD  ->      | TXD  ->      | TXD  -> | DIN
 * SRXD         | RXD  <-      | RXD  <-      | RXD  <- | DOUT
 *
 */

#define PORT1_SSI0	0
#define PORT2_SSI1	1
#define PORT3_PIN3	2
#define PORT4_PIN4	3
#define PORT5_CON11	4
#define PORT6_CON9	5
#define PORT7_PIN7	6

static void armadillo400_dam_init(void)
{
	unsigned int ptcr[7] = {0, 0, 0, 0, 0, 0, 0};
	unsigned int pdcr[7] = {0, 0, 0, 0, 0, 0, 0};
	int port;

#if defined(CONFIG_ARMADILLO400_AUD5_CON11_WM8978)
	/* SSI0 <-> PORT5(CON11) */
	ptcr[PORT1_SSI0] = \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0);

	ptcr[PORT5_CON11] = \
	MXC_AUDMUX_V2_PTCR_TFSDIR | \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_TCLKDIR | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RFSDIR | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RCLKDIR | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0);

	pdcr[PORT1_SSI0] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT5_CON11);
	pdcr[PORT5_CON11] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT1_SSI0);
#endif

#if defined(CONFIG_ARMADILLO400_AUD6_CON9_WM8978)
	/* SSI0 <-> PORT6(CON9) */
	ptcr[PORT1_SSI0] = \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0);

	ptcr[PORT6_CON9] = \
	MXC_AUDMUX_V2_PTCR_TFSDIR | \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_TCLKDIR | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RFSDIR | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0) | \
	MXC_AUDMUX_V2_PTCR_RCLKDIR | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT1_SSI0);

	pdcr[PORT1_SSI0] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT6_CON9);
	pdcr[PORT6_CON9] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT1_SSI0);
#endif

#if 0
	/* SSI1 <-> PORT6(CON9) */
	ptcr[PORT2_SSI1] = \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT2_SSI1);

	ptcr[PORT6_CON9] = \
	MXC_AUDMUX_V2_PTCR_TFSDIR | \
	MXC_AUDMUX_V2_PTCR_TFSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_TCLKDIR | \
	MXC_AUDMUX_V2_PTCR_TCSELa(MXC_AUDMUX_V2_FROM_TXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_RFSDIR | \
	MXC_AUDMUX_V2_PTCR_RFSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT2_SSI1) | \
	MXC_AUDMUX_V2_PTCR_RCLKDIR | \
	MXC_AUDMUX_V2_PTCR_RCSELa(MXC_AUDMUX_V2_FROM_RXFS, PORT2_SSI1);

	pdcr[PORT2_SSI1] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT6_CON9);
	pdcr[PORT6_CON9] = MXC_AUDMUX_V2_PDCR_RXDSEL(PORT2_SSI1);
#endif

	for (port = 0; port < 7; port++) {
		writel(ptcr[port], DAM_PTCR(port));
		writel(pdcr[port], DAM_PDCR(port));
		pr_debug("* DAM @ Port%d. PTCR(%p): 0x%08x, PDCR(%p): 0x%08x\n",
			 port + 1,
			 DAM_PTCR(port), ptcr[port],
			 DAM_PDCR(port), pdcr[port]);
	}
}

static int armadillo400_wm8978_audio_init(struct platform_device *pdev)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_audio_pads);
	armadillo400_dam_init();
	return 0;
}

static struct armadillo400_wm8978_audio_data armadillo400_wm8978_audio_data = {
	.init = armadillo400_wm8978_audio_init,
};

#if defined(CONFIG_ARMADILLO400_AUD5_CON11_WM8978)
static struct platform_device armadillo400_con11_wm8978_audio_device = {
	.name = "armadillo400-wm8978-audio",
	.id = 0,
};
#endif
#if defined(CONFIG_ARMADILLO400_AUD6_CON9_WM8978)
static struct platform_device armadillo400_con9_wm8978_audio_device = {
	.name = "armadillo400-wm8978-audio",
	.id = 0,
};
#endif

void __init armadillo400_audio_init(void)
{
#if defined(CONFIG_ARMADILLO400_AUD5_CON11_WM8978)
	mxc_register_device(&armadillo400_con11_wm8978_audio_device,
			    &armadillo400_wm8978_audio_data);
#endif
#if defined(CONFIG_ARMADILLO400_AUD6_CON9_WM8978)
	mxc_register_device(&armadillo400_con9_wm8978_audio_device,
			    &armadillo400_wm8978_audio_data);
#endif
}
