/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/pwm_backlight.h>
#include <mach/common.h>
#include <mach/mx2fb.h>
#include "armadillo400_iomux.h"

static struct pad_desc armadillo400_lcdc_pads[] = {
	MX25_PAD_LD0__LD0(0),
	MX25_PAD_LD1__LD1(0),
	MX25_PAD_LD2__LD2(0),
	MX25_PAD_LD3__LD3(0),
	MX25_PAD_LD4__LD4(0),
	MX25_PAD_LD5__LD5(0),
	MX25_PAD_LD6__LD6(0),
	MX25_PAD_LD7__LD7(0),
	MX25_PAD_LD8__LD8(0),
	MX25_PAD_LD9__LD9(0),
	MX25_PAD_LD10__LD10(0),
	MX25_PAD_LD11__LD11(0),
	MX25_PAD_LD12__LD12(0),
	MX25_PAD_LD13__LD13(0),
	MX25_PAD_LD14__LD14(0),
	MX25_PAD_LD15__LD15(0),
	MX25_PAD_GPIO_E__LD16(0),
	MX25_PAD_GPIO_F__LD17(0),
	MX25_PAD_HSYNC__HSYNC(0),
	MX25_PAD_VSYNC__VSYNC(0),
	MX25_PAD_LSCLK__LSCLK(PAD_CTL_SRE_FAST),
	MX25_PAD_OE_ACD__OE_ACD(0),
	MX25_PAD_CTL_GRP_DSE_LCD(PAD_CTL_DSE_LOW),
};

extern void imx_adc_set_hsync(int on) __attribute__((weak));

static int platform_fb_init(struct device *dev)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_lcdc_pads);
	imx_adc_set_hsync(1);

	return 0;
}

static int platform_fb_exit(struct device *dev)
{
	imx_adc_set_hsync(0);

	return 0;
}

static struct platform_pwm_backlight_data armadillo440_pwm_backlight_data = {
	.pwm_id = 0,
	.max_brightness = 255,
	.dft_brightness = 255,
	.pwm_period_ns = 10*1000*1000,
};

static struct platform_device armadillo400_pwm_backlight_device = {
	.name = "pwm-backlight",
};

static struct mx2fb_platform_data armadillo400_fb_data = {
	.fb_mode = "FG040360DSSWBG03",
	.regulator = "lcd-power",

	.init = platform_fb_init,
	.exit = platform_fb_exit,
};

static struct platform_device mx25_mxcfb_device = {
	.name = "mxc_sdc_fb",
	.id = 0,
	.dev = {
		.coherent_dma_mask = 0xFFFFFFFF,
	},
};

void __init armadillo400_fb_init(void)
{
	mxc_register_device(&mx25_mxcfb_device, &armadillo400_fb_data);
	mxc_register_device(&armadillo400_pwm_backlight_device,
			    &armadillo440_pwm_backlight_data);
}
