/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <mach/common.h>
#include "armadillo400_iomux.h"

struct armadillo400_ext_gpio {
	u32 gpio;
	int default_mode;
};

static struct pad_desc armadillo400_ext_gpio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_21_GPIO1_8)
	MX25_PAD_CSI_MCLK__GPIO_1_8(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_GPIO1_9)
	MX25_PAD_CSI_VSYNC__GPIO_1_9(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_GPIO1_10)
	MX25_PAD_CSI_HSYNC__GPIO_1_10(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_GPIO1_11)
	MX25_PAD_CSI_PIXCLK__GPIO_1_11(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_25_GPIO1_16)
	MX25_PAD_CSPI1_SS0__GPIO_1_16(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_26_GPIO)
	MX25_PAD_CSPI1_RDY__GPIO_2_22(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#elif defined(CONFIG_ARMADILLO400_CON9_26_W1)
	MX25_PAD_CSPI1_RDY__GPIO_2_22(PAD_CTL_PKE | PAD_CTL_PUS_22K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_27_GPIO2_21)
	MX25_PAD_CLKO__GPIO_2_21(0), /* no pull up */
#endif
#if defined(CONFIG_ARMADILLO400_CON9_28_GPIO3_15)
	MX25_PAD_EXT_ARMCLK__GPIO_3_15(), /* no pull up */
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_GPIO1_17)
	MX25_PAD_CSPI1_SS1__GPIO_1_17(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_GPIO1_29)
	MX25_PAD_CSI_D4__GPIO_1_29(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_GPIO1_18)
	MX25_PAD_CSPI1_SCLK__GPIO_1_18(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_GPIO1_30)
	MX25_PAD_CSI_D5__GPIO_1_30(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_GPIO1_7)
	MX25_PAD_CSI_D8__GPIO_1_7(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_16_GPIO1_31)
	MX25_PAD_CSI_D6__GPIO_1_31(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_GPIO4_21)
	MX25_PAD_CSI_D9__GPIO_4_21(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_18_GPIO1_6)
	MX25_PAD_CSI_D7__GPIO_1_6(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_1_GPIO3_17) || defined(CONFIG_ARMADILLO400_CON9_1_SDHC2_PWREN)
	MX25_PAD_VSTBY_REQ__GPIO_3_17(PAD_CTL_PKE),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_2_GPIO3_14)
	MX25_PAD_RTCK__GPIO_3_14(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_GPIO_2_31)
	MX25_PAD_KPP_ROW2__GPIO_2_31(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_GPIO_3_0)
	MX25_PAD_KPP_ROW3__GPIO_3_0(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_44_GPIO_3_1)
	MX25_PAD_KPP_COL0__GPIO_3_1(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_GPIO_3_2)
	MX25_PAD_KPP_COL1__GPIO_3_2(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_GPIO_3_3)
	MX25_PAD_KPP_COL2__GPIO_3_3(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_GPIO_3_4)
	MX25_PAD_KPP_COL3__GPIO_3_4(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_48_GPIO1_0)
	MX25_PAD_GPIO_A__GPIO_A(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_GPIO1_1)
	MX25_PAD_GPIO_B__GPIO_B(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_3_GPIO1_2)
	MX25_PAD_GPIO_C__GPIO_C(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_GPIO1_3)
	MX25_PAD_GPIO_D__GPIO_D(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_3_GPIO1_14)
	MX25_PAD_CSPI1_MOSI__GPIO_1_14(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_4_GPIO1_27)
	MX25_PAD_CSI_D2__GPIO_1_27(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_GPIO1_15)
	MX25_PAD_CSPI1_MISO__GPIO_1_15(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_GPIO1_28)
	MX25_PAD_CSI_D3__GPIO_1_28(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

static struct armadillo400_ext_gpio armadillo400_gpio_list[] = {
#if defined(CONFIG_ARMADILLO400_CON9_21_GPIO1_8)
	{ GPIO(1, 8), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_GPIO1_9)
	{ GPIO(1, 9), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_GPIO1_10)
	{ GPIO(1, 10), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_GPIO1_11)
	{ GPIO(1, 11), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_25_GPIO1_16)
	{ GPIO(1, 16), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_26_GPIO)
	{ GPIO(2, 22), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_27_GPIO2_21)
	{ GPIO(2, 21), GPIOF_OUT_INIT_LOW },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_28_GPIO3_15)
	{ GPIO(3, 15), GPIOF_OUT_INIT_LOW },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_GPIO1_17)
	{ GPIO(1, 17), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_GPIO1_29)
	{ GPIO(1, 29), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_GPIO1_18)
	{ GPIO(1, 18), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_GPIO1_30)
	{ GPIO(1, 30), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_GPIO1_7)
	{ GPIO(1, 7), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_16_GPIO1_31)
	{ GPIO(1, 31), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_GPIO4_21)
	{ GPIO(4, 21), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_18_GPIO1_6)
	{ GPIO(1, 6), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_1_GPIO3_17)
	{ GPIO(3, 17), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_2_GPIO3_14)
	{ GPIO(3, 14), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_GPIO_2_31)
	{ GPIO(2, 31), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_GPIO_3_0)
	{ GPIO(3, 0), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_44_GPIO_3_1)
	{ GPIO(3, 1), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_GPIO_3_2)
	{ GPIO(3, 2), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_GPIO_3_3)
	{ GPIO(3, 3), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_GPIO_3_4)
	{ GPIO(3, 4), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_48_GPIO1_0)
	{ GPIO(1, 0), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_GPIO1_1)
	{ GPIO(1, 1), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON14_3_GPIO1_2)
	{ GPIO(1, 2), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_GPIO1_3)
	{ GPIO(1, 3), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_3_GPIO1_14)
	{ GPIO(1, 14), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_4_GPIO1_27)
	{ GPIO(1, 27), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_GPIO1_15)
	{ GPIO(1, 15), GPIOF_IN },
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_GPIO1_28)
	{ GPIO(1, 28), GPIOF_IN },
#endif
};

const char *mxc_gpio_names[128] = {
	[GPIO(3, 17)] = "CON9_1",
	[GPIO(3, 14)] = "CON9_2",
	[GPIO(1, 14)] = "CON9_3",
	[GPIO(1, 27)] = "CON9_4",
	[GPIO(1, 15)] = "CON9_5",
	[GPIO(1, 28)] = "CON9_6",
	[GPIO(1, 17)] = "CON9_11",
	[GPIO(1, 29)] = "CON9_12",
	[GPIO(1, 18)] = "CON9_13",
	[GPIO(1, 30)] = "CON9_14",
	[GPIO(1,  7)] = "CON9_15",
	[GPIO(1, 31)] = "CON9_16",
	[GPIO(4, 21)] = "CON9_17",
	[GPIO(1,  6)] = "CON9_18",
	[GPIO(1,  8)] = "CON9_21",
	[GPIO(1,  9)] = "CON9_22",
	[GPIO(1, 10)] = "CON9_23",
	[GPIO(1, 11)] = "CON9_24",
	[GPIO(1, 16)] = "CON9_25",
	[GPIO(2, 22)] = "CON9_26",
	[GPIO(2, 21)] = "CON9_27",
	[GPIO(3, 15)] = "CON9_28",

	[GPIO(2, 31)] = "CON11_42",
	[GPIO(3,  0)] = "CON11_43",
	[GPIO(3,  1)] = "CON11_44",
	[GPIO(3,  2)] = "CON11_45",
	[GPIO(3,  3)] = "CON11_46",
	[GPIO(3,  4)] = "CON11_47",
	[GPIO(1,  0)] = "CON11_48",
	[GPIO(1,  1)] = "CON11_49",

	[GPIO(1,  2)] = "CON14_3",
	[GPIO(1,  3)] = "CON14_4",
};

void __init armadillo400_gpio_init(void)
{
	struct armadillo400_ext_gpio *gpios = armadillo400_gpio_list;
	int i;

	for (i = 0; i < ARRAY_SIZE(armadillo400_gpio_list); i++) {
		gpio_request(gpios[i].gpio, mxc_gpio_names[gpios[i].gpio]);
		gpio_export(gpios[i].gpio, true);
		if (gpios[i].default_mode == GPIOF_IN) {
			gpio_direction_input(gpios[i].gpio);
		} else if (gpios[i].default_mode == GPIOF_OUT_INIT_LOW) {
			gpio_direction_output(gpios[i].gpio, 0);
		} else {
			gpio_direction_output(gpios[i].gpio, 1);
		}
	}

	mxc_iomux_v3a_setup_multiple_pads(armadillo400_ext_gpio_pads);
}
